@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Manjeet Dahiya
@@ -184,6 +184,8 @@ extern const char* ns_omemo;
 extern const char* ns_mix_pam;
 extern const char* ns_mix_roster;
 extern const char* ns_mix_presence;
+// XEP-0407: Mediated Information eXchange (MIX): Miscellaneous Capabilities
+extern const char* ns_mix_misc;
 // XEP-0428: Fallback Indication
 extern const char* ns_fallback_indication;
 
