@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Jeremy Lainé
@@ -828,7 +828,7 @@ void QXmppDataForm::toXml(QXmlStreamWriter *writer) const
                    type == QXmppDataForm::Field::TextMultiField) {
             for (const QString &value : field.value().toStringList())
                 helperToXmlAddTextElement(writer, "value", value);
-        } else if (!field.value().isNull()) {
+        } else if (!field.value().toString().isEmpty()) {
             helperToXmlAddTextElement(writer, "value", field.value().toString());
         }
 
