@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Manjeet Dahiya
@@ -26,13 +26,16 @@
 #ifndef QXMPPMESSAGE_H
 #define QXMPPMESSAGE_H
 
-// Required for source compatibility
 #include "QXmppStanza.h"
 
+// Required for source compatibility
 #include <QDateTime>
 
+#include <optional>
+
 class QXmppMessagePrivate;
 class QXmppBitsOfBinaryDataList;
+class QXmppMixInvitation;
 
 ///
 /// \brief The QXmppMessage class represents an XMPP message.
@@ -243,6 +246,10 @@ public:
     QString spoilerHint() const;
     void setSpoilerHint(const QString &);
 
+    // XEP-0407: Mediated Information eXchange (MIX): Miscellaneous Capabilities
+    std::optional<QXmppMixInvitation> mixInvitation() const;
+    void setMixInvitation(const std::optional<QXmppMixInvitation> &mixInvitation);
+
     // XEP-0428: Fallback Indication
     bool isFallback() const;
     void setIsFallback(bool isFallback);
@@ -259,4 +266,6 @@ private:
     QSharedDataPointer<QXmppMessagePrivate> d;
 };
 
+Q_DECLARE_METATYPE(QXmppMessage)
+
 #endif  // QXMPPMESSAGE_H
