@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Manjeet Dahiya
@@ -447,7 +447,11 @@ void QXmppPresence::parseExtension(const QDomElement &element, QXmppElementList
         d->capabilityNode = element.attribute(QStringLiteral("node"));
         d->capabilityVer = QByteArray::fromBase64(element.attribute(QStringLiteral("ver")).toLatin1());
         d->capabilityHash = element.attribute(QStringLiteral("hash"));
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+        d->capabilityExt = element.attribute(QStringLiteral("ext")).split(' ', Qt::SkipEmptyParts);
+#else
         d->capabilityExt = element.attribute(QStringLiteral("ext")).split(' ', QString::SkipEmptyParts);
+#endif
         // XEP-0153: vCard-Based Avatars
     } else if (element.namespaceURI() == ns_vcard_update) {
         QDomElement photoElement = element.firstChildElement(QStringLiteral("photo"));
