@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Authors:
  *  Manjeet Dahiya
@@ -34,7 +34,7 @@
 #include <QDomDocument>
 #include <QHostAddress>
 #include <QMap>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QSslSocket>
 #include <QStringList>
 #include <QTime>
@@ -43,27 +43,25 @@
 #if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
 static bool randomSeeded = false;
 #endif
-static const QByteArray streamRootElementEnd = QByteArrayLiteral("</stream:stream>");
 
 class QXmppStreamPrivate
 {
 public:
-    QXmppStreamPrivate();
+    QXmppStreamPrivate(QXmppStream *stream);
 
-    QByteArray dataBuffer;
+    QString dataBuffer;
     QSslSocket *socket;
 
     // incoming stream state
-    QByteArray streamStart;
+    QString streamOpenElement;
 
-    bool streamManagementEnabled;
-    QMap<unsigned, QByteArray> unacknowledgedStanzas;
-    unsigned lastOutgoingSequenceNumber;
-    unsigned lastIncomingSequenceNumber;
+    // stream management
+    QXmppStreamManager streamManager;
 };
 
-QXmppStreamPrivate::QXmppStreamPrivate()
-    : socket(nullptr), streamManagementEnabled(false), lastOutgoingSequenceNumber(0), lastIncomingSequenceNumber(0)
+QXmppStreamPrivate::QXmppStreamPrivate(QXmppStream *stream)
+    : socket(nullptr),
+      streamManager(stream)
 {
 }
 
@@ -74,7 +72,7 @@ QXmppStreamPrivate::QXmppStreamPrivate()
 ///
 QXmppStream::QXmppStream(QObject *parent)
     : QXmppLoggable(parent),
-      d(new QXmppStreamPrivate)
+      d(new QXmppStreamPrivate(this))
 {
 #if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
     // Make sure the random number generator is seeded
@@ -98,10 +96,11 @@ QXmppStream::~QXmppStream()
 ///
 void QXmppStream::disconnectFromHost()
 {
-    d->streamManagementEnabled = false;
+    d->streamManager.handleDisconnect();
+
     if (d->socket) {
         if (d->socket->state() == QAbstractSocket::ConnectedState) {
-            sendData(streamRootElementEnd);
+            sendData(QByteArrayLiteral("</stream:stream>"));
             d->socket->flush();
         }
         // FIXME: according to RFC 6120 section 4.4, we should wait for
@@ -118,9 +117,9 @@ void QXmppStream::disconnectFromHost()
 ///
 void QXmppStream::handleStart()
 {
-    d->streamManagementEnabled = false;
+    d->streamManager.handleStart();
     d->dataBuffer.clear();
-    d->streamStart.clear();
+    d->streamOpenElement.clear();
 }
 
 ///
@@ -157,17 +156,28 @@ bool QXmppStream::sendPacket(const QXmppStanza &packet)
     QXmlStreamWriter xmlStream(&data);
     packet.toXml(&xmlStream);
 
-    bool isXmppStanza = packet.isXmppStanza();
-    if (isXmppStanza && d->streamManagementEnabled)
-        d->unacknowledgedStanzas[++d->lastOutgoingSequenceNumber] = data;
-
     // send packet
     bool success = sendData(data);
-    if (isXmppStanza)
-        sendAcknowledgementRequest();
+
+    // handle stream management
+    d->streamManager.handlePacketSent(packet, data);
+
     return success;
 }
 
+///
+/// Resets the stream management packages cache.
+///
+/// This can be done to prevent that packages from the last connection are being
+/// resent.
+///
+/// \since QXmpp 1.4
+///
+void QXmppStream::resetPacketCache()
+{
+    d->streamManager.resetCache();
+}
+
 ///
 /// Returns the QSslSocket used for this stream.
 ///
@@ -188,7 +198,11 @@ void QXmppStream::setSocket(QSslSocket *socket)
     // socket events
     connect(socket, &QAbstractSocket::connected, this, &QXmppStream::_q_socketConnected);
     connect(socket, &QSslSocket::encrypted, this, &QXmppStream::_q_socketEncrypted);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    connect(socket, &QSslSocket::errorOccurred, this, &QXmppStream::_q_socketError);
+#else
     connect(socket, QOverload<QAbstractSocket::SocketError>::of(&QSslSocket::error), this, &QXmppStream::_q_socketError);
+#endif
     connect(socket, &QIODevice::readyRead, this, &QXmppStream::_q_socketReadyRead);
 }
 
@@ -212,72 +226,112 @@ void QXmppStream::_q_socketError(QAbstractSocket::SocketError socketError)
 
 void QXmppStream::_q_socketReadyRead()
 {
-    d->dataBuffer.append(d->socket->readAll());
+    processData(QString::fromUtf8(d->socket->readAll()));
+}
 
-    // handle whitespace pings
-    if (!d->dataBuffer.isEmpty() && d->dataBuffer.trimmed().isEmpty()) {
+void QXmppStream::processData(const QString &data)
+{
+    // As we may only have partial XML content, we need to cache the received
+    // data until it has been successfully parsed. In case it can't be parsed,
+    //
+    // There are only two small problems with the current strategy:
+    //  * When we receive a full stanza + a partial one, we can't parse the
+    //    first stanza until another stanza arrives that is complete.
+    //  * We don't know when we received invalid XML (would cause a growing
+    //    cache and a timeout after some time).
+    // However, both issues could only be solved using an XML stream reader
+    // which would cause many other problems since we don't actually use it for
+    // parsing the content.
+    d->dataBuffer.append(data);
+
+    //
+    // Check for whitespace pings
+    //
+    if (d->dataBuffer.isEmpty() || d->dataBuffer.trimmed().isEmpty()) {
         d->dataBuffer.clear();
-        handleStanza(QDomElement());
+
+        logReceived({});
+        handleStanza({});
+        return;
     }
 
-    // FIXME : maybe these QRegExps could be static?
-    QRegExp startStreamRegex(R"(^(<\?xml.*\?>)?\s*<stream:stream.*>)");
-    startStreamRegex.setMinimal(true);
-    QRegExp endStreamRegex("</stream:stream>$");
-    endStreamRegex.setMinimal(true);
-
-    // check whether we need to add stream start / end elements
     //
-    // NOTE: as we may only have partial XML content, do not alter the stream's
-    // state until we have a valid XML document!
-    QByteArray completeXml = d->dataBuffer;
-    const QString strData = QString::fromUtf8(d->dataBuffer);
-    bool streamStart = false;
-    if (d->streamStart.isEmpty() && strData.contains(startStreamRegex))
-        streamStart = true;
-    else
-        completeXml.prepend(d->streamStart);
-    bool streamEnd = false;
-    if (strData.contains(endStreamRegex))
-        streamEnd = true;
-    else
-        completeXml.append(streamRootElementEnd);
+    // Check whether we received a stream open or closing tag
+    //
+    static const QRegularExpression streamStartRegex(R"(^(<\?xml.*\?>)?\s*<stream:stream[^>]*>)");
+    static const QRegularExpression streamEndRegex("</stream:stream>$");
 
-    // check whether we have a valid XML document
+    QRegularExpressionMatch streamOpenMatch;
+    bool hasStreamOpen = d->streamOpenElement.isEmpty() &&
+        (streamOpenMatch = streamStartRegex.match(d->dataBuffer)).hasMatch();
+
+    bool hasStreamClose = streamEndRegex.match(d->dataBuffer).hasMatch();
+
+    //
+    // The stream start/end and stanza packets can't be parsed without any
+    // modifications with QDomDocument. This is because of multiple reasons:
+    //  * The <stream:stream> open element is not considered valid without the
+    //    closing tag.
+    //  * Only the closing tag is of course not valid too.
+    //  * Stanzas/Nonzas need to have the correct stream namespaces set:
+    //     * For being able to parse <stream:features/>
+    //     * For having the correct namespace (e.g. 'jabber:client') set to
+    //       stanzas and their child elements (e.g. <body/> of a message).
+    //
+    // The wrapping strategy looks like this:
+    //  * The stream open tag is cached once it arrives, for later access
+    //  * Incoming XML that has no <stream> open tag will be prepended by the
+    //    cached <stream> tag.
+    //  * Incoming XML that has no <stream> close tag will be appended by a
+    //    generic string "</stream:stream>"
+    //
+    // The result is parsed by QDomDocument and the child elements of the stream
+    // are processed. In case the received data contained a stream open tag,
+    // the stream is processed (before the stanzas are processed). In case we
+    // received a </stream> closing tag, the connection is closed.
+    //
+    auto wrappedStanzas = d->dataBuffer;
+    if (!hasStreamOpen) {
+        wrappedStanzas.prepend(d->streamOpenElement);
+    }
+    if (!hasStreamClose) {
+        wrappedStanzas.append(QStringLiteral("</stream:stream>"));
+    }
+
+    //
+    // Try to parse the wrapped XML
+    //
     QDomDocument doc;
-    if (!doc.setContent(completeXml, true))
+    if (!doc.setContent(wrappedStanzas, true))
         return;
 
-    // remove data from buffer
-    logReceived(strData);
+    //
+    // Success: We can clear the buffer and send a 'received' log message
+    //
+    logReceived(d->dataBuffer);
     d->dataBuffer.clear();
 
     // process stream start
-    if (streamStart) {
-        d->streamStart = startStreamRegex.cap(0).toUtf8();
+    if (hasStreamOpen) {
+        d->streamOpenElement = streamOpenMatch.captured();
         handleStream(doc.documentElement());
     }
 
     // process stanzas
-    QDomElement nodeRecv = doc.documentElement().firstChildElement();
-    while (!nodeRecv.isNull()) {
-        if (QXmppStreamManagementAck::isStreamManagementAck(nodeRecv))
-            handleAcknowledgement(nodeRecv);
-        else if (QXmppStreamManagementReq::isStreamManagementReq(nodeRecv))
-            sendAcknowledgement();
-        else {
-            handleStanza(nodeRecv);
-            if (nodeRecv.tagName() == QLatin1String("message") ||
-                nodeRecv.tagName() == QLatin1String("presence") ||
-                nodeRecv.tagName() == QLatin1String("iq"))
-                ++d->lastIncomingSequenceNumber;
-        }
-        nodeRecv = nodeRecv.nextSiblingElement();
+    auto stanza = doc.documentElement().firstChildElement();
+    for (; !stanza.isNull(); stanza = stanza.nextSiblingElement()) {
+        // handle possible stream management packets first
+        if (d->streamManager.handleStanza(stanza))
+            continue;
+
+        // process all other kinds of packets
+        handleStanza(stanza);
     }
 
     // process stream end
-    if (streamEnd)
+    if (hasStreamClose) {
         disconnectFromHost();
+    }
 }
 
 ///
@@ -290,30 +344,7 @@ void QXmppStream::_q_socketReadyRead()
 ///
 void QXmppStream::enableStreamManagement(bool resetSequenceNumber)
 {
-    d->streamManagementEnabled = true;
-
-    if (resetSequenceNumber) {
-        d->lastOutgoingSequenceNumber = 0;
-        d->lastIncomingSequenceNumber = 0;
-
-        // resend unacked stanzas
-        if (!d->unacknowledgedStanzas.empty()) {
-            QMap<unsigned, QByteArray> oldUnackedStanzas = d->unacknowledgedStanzas;
-            d->unacknowledgedStanzas.clear();
-            for (QMap<unsigned, QByteArray>::iterator it = oldUnackedStanzas.begin(); it != oldUnackedStanzas.end(); ++it) {
-                d->unacknowledgedStanzas[++d->lastOutgoingSequenceNumber] = it.value();
-                sendData(it.value());
-            }
-            sendAcknowledgementRequest();
-        }
-    } else {
-        // resend unacked stanzas
-        if (!d->unacknowledgedStanzas.empty()) {
-            for (QMap<unsigned, QByteArray>::iterator it = d->unacknowledgedStanzas.begin(); it != d->unacknowledgedStanzas.end(); ++it)
-                sendData(it.value());
-            sendAcknowledgementRequest();
-        }
-    }
+    d->streamManager.enableStreamManagement(resetSequenceNumber);
 }
 
 ///
@@ -321,9 +352,9 @@ void QXmppStream::enableStreamManagement(bool resetSequenceNumber)
 ///
 /// \since QXmpp 1.0
 ///
-unsigned QXmppStream::lastIncomingSequenceNumber() const
+unsigned int QXmppStream::lastIncomingSequenceNumber() const
 {
-    return d->lastIncomingSequenceNumber;
+    return d->streamManager.lastIncomingSequenceNumber();
 }
 
 ///
@@ -332,68 +363,7 @@ unsigned QXmppStream::lastIncomingSequenceNumber() const
 ///
 /// \since QXmpp 1.0
 ///
-void QXmppStream::setAcknowledgedSequenceNumber(unsigned sequenceNumber)
+void QXmppStream::setAcknowledgedSequenceNumber(unsigned int sequenceNumber)
 {
-    for (QMap<unsigned, QByteArray>::iterator it = d->unacknowledgedStanzas.begin(); it != d->unacknowledgedStanzas.end();) {
-        if (it.key() <= sequenceNumber)
-            it = d->unacknowledgedStanzas.erase(it);
-        else
-            ++it;
-    }
-}
-
-///
-/// Handles an incoming acknowledgement from \xep{0198}.
-///
-/// \param element
-///
-/// \since QXmpp 1.0
-///
-void QXmppStream::handleAcknowledgement(QDomElement &element)
-{
-    if (!d->streamManagementEnabled)
-        return;
-
-    QXmppStreamManagementAck ack;
-    ack.parse(element);
-    setAcknowledgedSequenceNumber(ack.seqNo());
-}
-
-///
-/// Sends an acknowledgement as defined in \xep{0198}.
-///
-/// \since QXmpp 1.0
-///
-void QXmppStream::sendAcknowledgement()
-{
-    if (!d->streamManagementEnabled)
-        return;
-
-    // prepare packet
-    QByteArray data;
-    QXmlStreamWriter xmlStream(&data);
-    QXmppStreamManagementAck ack(d->lastIncomingSequenceNumber);
-    ack.toXml(&xmlStream);
-
-    // send packet
-    sendData(data);
-}
-
-///
-/// Sends an acknowledgement request as defined in \xep{0198}.
-///
-/// \since QXmpp 1.0
-///
-void QXmppStream::sendAcknowledgementRequest()
-{
-    if (!d->streamManagementEnabled)
-        return;
-
-    // prepare packet
-    QByteArray data;
-    QXmlStreamWriter xmlStream(&data);
-    QXmppStreamManagementReq::toXml(&xmlStream);
-
-    // send packet
-    sendData(data);
+    d->streamManager.setAcknowledgedSequenceNumber(sequenceNumber);
 }
