@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Authors:
  *  Manjeet Dahiya
@@ -35,9 +35,9 @@ class QSslSocket;
 class QXmppStanza;
 class QXmppStreamPrivate;
 
+///
 /// \brief The QXmppStream class is the base class for all XMPP streams.
 ///
-
 class QXMPP_EXPORT QXmppStream : public QXmppLoggable
 {
     Q_OBJECT
@@ -49,6 +49,8 @@ public:
     virtual bool isConnected() const;
     bool sendPacket(const QXmppStanza &);
 
+    void resetPacketCache();
+
 Q_SIGNALS:
     /// This signal is emitted when the stream is connected.
     void connected();
@@ -76,14 +78,8 @@ protected:
 
     // XEP-0198: Stream Management
     void enableStreamManagement(bool resetSequenceNumber);
-    unsigned lastIncomingSequenceNumber() const;
-    void setAcknowledgedSequenceNumber(unsigned sequenceNumber);
-
-private:
-    // XEP-0198: Stream Management
-    void handleAcknowledgement(QDomElement &element);
-    void sendAcknowledgement();
-    void sendAcknowledgementRequest();
+    unsigned int lastIncomingSequenceNumber() const;
+    void setAcknowledgedSequenceNumber(unsigned int sequenceNumber);
 
 public Q_SLOTS:
     virtual void disconnectFromHost();
@@ -96,6 +92,10 @@ private Q_SLOTS:
     void _q_socketReadyRead();
 
 private:
+    void processData(const QString &data);
+
+    friend class tst_QXmppStream;
+
     QXmppStreamPrivate *const d;
 };
 
