@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Jeremy Lainé
@@ -24,16 +24,12 @@
 #ifndef QXMPPARCHIVEMANAGER_H
 #define QXMPPARCHIVEMANAGER_H
 
+#include "QXmppArchiveIq.h"
 #include "QXmppClientExtension.h"
 #include "QXmppResultSet.h"
 
 #include <QDateTime>
 
-class QXmppArchiveChat;
-class QXmppArchiveChatIq;
-class QXmppArchiveListIq;
-class QXmppArchivePrefIq;
-
 /// \brief The QXmppArchiveManager class makes it possible to access message
 /// archives as defined by \xep{0136}: Message Archiving.
 ///
