@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Manjeet Dahiya
@@ -86,8 +86,6 @@ QStringList QXmppClientPrivate::discoveryFeatures()
         ns_capabilities,
         // XEP-0199: XMPP Ping
         ns_ping,
-        // XEP-0224: Attention
-        ns_attention,
         // XEP-0249: Direct MUC Invitations
         ns_conference,
         // XEP-0308: Last Message Correction
@@ -234,17 +232,23 @@ QXmppConfiguration& QXmppClient::configuration()
     return d->stream->configuration();
 }
 
+///
 /// Attempts to connect to the XMPP server. Server details and other configurations
 /// are specified using the config parameter. Use signals connected(), error(QXmppClient::Error)
 /// and disconnected() to know the status of the connection.
+///
 /// \param config Specifies the configuration object for connecting the XMPP server.
 /// This contains the host name, user, password etc. See QXmppConfiguration for details.
 /// \param initialPresence The initial presence which will be set for this user
 /// after establishing the session. The default value is QXmppPresence::Available
-
+///
 void QXmppClient::connectToServer(const QXmppConfiguration& config,
                                   const QXmppPresence& initialPresence)
 {
+    // reset package cache from last connection
+    if (d->stream->configuration().jidBare() != config.jidBare())
+        d->stream->resetPacketCache();
+
     d->stream->configuration() = config;
     d->clientPresence = initialPresence;
     d->addProperCapability(d->clientPresence);
@@ -352,6 +356,24 @@ void QXmppClient::setActive(bool active)
     }
 }
 
+///
+/// Returns the current \xep{0198}: Stream Management state of the connection.
+///
+/// Upon connection of the client this can be used to check whether the
+/// previous stream has been resumed.
+///
+/// \since QXmpp 1.4
+///
+QXmppClient::StreamManagementState QXmppClient::streamManagementState() const
+{
+    if (d->stream->isStreamManagementEnabled()) {
+        if (d->stream->isStreamResumed())
+            return ResumedStream;
+        return NewStream;
+    }
+    return NoStreamManagement;
+}
+
 /// Returns the reference to QXmppRosterManager object of the client.
 ///
 /// \return Reference to the roster object of the connected client. Use this to
