@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Manjeet Dahiya
@@ -30,8 +30,7 @@
 
 #include <QAbstractSocket>
 #include <QObject>
-
-class QSslError;
+#include <QSslError>
 
 class QXmppClientExtension;
 class QXmppClientPrivate;
@@ -74,13 +73,13 @@ class QXmppVersionManager;
 /// QXmppVersionManager (software version information).
 ///
 /// By default, the client will automatically try reconnecting to the server.
-/// You can change this a behaviour using
+/// You can change that behaviour using
 /// QXmppConfiguration::setAutoReconnectionEnabled().
 ///
 /// Not all the managers or extensions have been enabled by default. One can
 /// enable/disable the managers using the functions \c addExtension() and
-/// \c removeExtension(). \c findExtension() can be used to find reference/
-/// pointer to particular instansiated and enabled manager.
+/// \c removeExtension(). \c findExtension() can be used to find a
+/// reference/pointer to a particular instantiated and enabled manager.
 ///
 /// List of managers enabled by default:
 /// - QXmppRosterManager
@@ -90,7 +89,7 @@ class QXmppVersionManager;
 /// - QXmppEntityTimeManager
 ///
 /// \ingroup Core
-
+///
 class QXMPP_EXPORT QXmppClient : public QXmppLoggable
 {
     Q_OBJECT
@@ -119,6 +118,16 @@ public:
     };
     Q_ENUM(State)
 
+    /// Describes the use of \xep{0198}: Stream Management
+    enum StreamManagementState {
+        /// Stream Management is not used.
+        NoStreamManagement,
+        /// Stream Management is used and the previous stream has not been resumed.
+        NewStream,
+        /// Stream Management is used and the previous stream has been resumed.
+        ResumedStream
+    };
+
     QXmppClient(QObject *parent = nullptr);
     ~QXmppClient() override;
 
@@ -128,6 +137,7 @@ public:
 
     QList<QXmppClientExtension *> extensions();
 
+    ///
     /// \brief Returns the extension which can be cast into type T*, or 0
     /// if there is no such extension.
     ///
@@ -152,6 +162,7 @@ public:
         return nullptr;
     }
 
+    ///
     /// \brief Returns the index of an extension
     ///
     /// Usage example:
@@ -183,6 +194,8 @@ public:
     bool isActive() const;
     void setActive(bool active);
 
+    StreamManagementState streamManagementState() const;
+
     QXmppPresence clientPresence() const;
     void setClientPresence(const QXmppPresence &presence);
 
@@ -261,10 +274,12 @@ Q_SIGNALS:
     /// changes Busy, Idle, Invisible etc.
     void presenceReceived(const QXmppPresence &presence);
 
-    /// Notifies that an XMPP iq stanza is received. The QXmppIq
-    /// parameter contains the details of the iq sent to this client.
-    /// IQ stanzas provide a structured request-response mechanism. Roster
-    /// management, setting-getting vCards etc is done using iq stanzas.
+    /// This signal is emitted when IQs of type result or error are received by
+    /// the client and no registered QXmppClientExtension could handle it.
+    ///
+    /// This is useful when it is only important to check whether the response
+    /// of an IQ was successful. However, the recommended way is still to use an
+    /// additional QXmppClientExtension for this kind of tasks.
     void iqReceived(const QXmppIq &iq);
 
     /// This signal is emitted to indicate that one or more SSL errors were
