@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Authors:
  *  Manjeet Dahiya
@@ -52,7 +52,7 @@
 #include <QCoreApplication>
 #include <QDomDocument>
 #include <QHostAddress>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QStringList>
 #include <QTimer>
 #include <QXmlStreamWriter>
@@ -107,6 +107,8 @@ public:
     bool isResuming;
     QString resumeHost;
     quint16 resumePort;
+    bool streamManagementEnabled;
+    bool streamResumed;
 
     // Client State Indication
     bool clientStateIndicationEnabled;
@@ -120,7 +122,23 @@ private:
 };
 
 QXmppOutgoingClientPrivate::QXmppOutgoingClientPrivate(QXmppOutgoingClient *qq)
-    : nextSrvRecordIdx(0), redirectPort(0), bindModeAvailable(false), sessionAvailable(false), sessionStarted(false), isAuthenticated(false), saslClient(nullptr), streamManagementAvailable(false), canResume(false), isResuming(false), resumePort(0), clientStateIndicationEnabled(false), pingTimer(nullptr), timeoutTimer(nullptr), q(qq)
+    : nextSrvRecordIdx(0),
+      redirectPort(0),
+      bindModeAvailable(false),
+      sessionAvailable(false),
+      sessionStarted(false),
+      isAuthenticated(false),
+      saslClient(nullptr),
+      streamManagementAvailable(false),
+      canResume(false),
+      isResuming(false),
+      resumePort(0),
+      streamManagementEnabled(false),
+      streamResumed(false),
+      clientStateIndicationEnabled(false),
+      pingTimer(nullptr),
+      timeoutTimer(nullptr),
+      q(qq)
 {
 }
 
@@ -176,7 +194,11 @@ QXmppOutgoingClient::QXmppOutgoingClient(QObject *parent)
 
     connect(socket, &QAbstractSocket::disconnected, this, &QXmppOutgoingClient::_q_socketDisconnected);
     connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &QXmppOutgoingClient::socketSslErrors);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    connect(socket, &QSslSocket::errorOccurred, this, &QXmppOutgoingClient::socketError);
+#else
     connect(socket, QOverload<QAbstractSocket::SocketError>::of(&QSslSocket::error), this, &QXmppOutgoingClient::socketError);
+#endif
 
     // DNS lookups
     connect(&d->dns, &QDnsLookup::finished, this, &QXmppOutgoingClient::_q_dnsLookupFinished);
@@ -281,6 +303,30 @@ bool QXmppOutgoingClient::isClientStateIndicationEnabled() const
     return d->clientStateIndicationEnabled;
 }
 
+///
+/// Returns whether Stream Management is currently enabled.
+///
+/// \since QXmpp 1.4
+///
+bool QXmppOutgoingClient::isStreamManagementEnabled() const
+{
+    return d->streamManagementEnabled;
+}
+
+///
+/// Returns true if the current stream is a successful resumption of a previous
+/// stream.
+///
+/// In case a stream has been resumed, some tasks like fetching the roster again
+/// are not required.
+///
+/// \since QXmpp 1.4
+///
+bool QXmppOutgoingClient::isStreamResumed() const
+{
+    return d->streamResumed;
+}
+
 void QXmppOutgoingClient::_q_socketDisconnected()
 {
     debug("Socket disconnected");
@@ -342,6 +388,10 @@ void QXmppOutgoingClient::handleStart()
     d->sessionAvailable = false;
     d->sessionStarted = false;
 
+    // reset stream management
+    d->streamResumed = false;
+    d->streamManagementEnabled = false;
+
     // start stream
     QByteArray data = "<?xml version='1.0'?><stream:stream to='";
     data.append(configuration().domain().toUtf8());
@@ -488,13 +538,8 @@ void QXmppOutgoingClient::handleStanza(const QDomElement &nodeRecv)
         emit connected();
     } else if (ns == ns_stream && nodeRecv.tagName() == "error") {
         // handle redirects
-        QRegExp redirectRegex("([^:]+)(:[0-9]+)?");
-        if (redirectRegex.exactMatch(nodeRecv.firstChildElement("see-other-host").text())) {
-            d->redirectHost = redirectRegex.cap(0);
-            if (!redirectRegex.cap(2).isEmpty())
-                d->redirectPort = redirectRegex.cap(2).mid(1).toUShort();
-            else
-                d->redirectPort = 5222;
+        const auto otherHost = nodeRecv.firstChildElement("see-other-host");
+        if (!otherHost.isNull() && setResumeAddress(otherHost.text())) {
             QXmppStream::disconnectFromHost();
             return;
         }
@@ -569,11 +614,12 @@ void QXmppOutgoingClient::handleStanza(const QDomElement &nodeRecv)
                 // bind result
                 if (bind.type() == QXmppIq::Result) {
                     if (!bind.jid().isEmpty()) {
-                        QRegExp jidRegex("^([^@/]+)@([^@/]+)/(.+)$");
-                        if (jidRegex.exactMatch(bind.jid())) {
-                            configuration().setUser(jidRegex.cap(1));
-                            configuration().setDomain(jidRegex.cap(2));
-                            configuration().setResource(jidRegex.cap(3));
+                        static const QRegularExpression jidRegex("^([^@/]+)@([^@/]+)/(.+)$");
+
+                        if (const auto match = jidRegex.match(bind.jid()); match.hasMatch()) {
+                            configuration().setUser(match.captured(1));
+                            configuration().setDomain(match.captured(2));
+                            configuration().setResource(match.captured(3));
                         } else {
                             warning("Bind IQ received with invalid JID: " + bind.jid());
                         }
@@ -591,6 +637,11 @@ void QXmppOutgoingClient::handleStanza(const QDomElement &nodeRecv)
                             emit connected();
                         }
                     }
+                } else if (bind.type() == QXmppIq::Error) {
+                    d->xmppStreamError = bind.error().condition();
+                    emit error(QXmppClient::XmppStreamError);
+                    warning("Resource binding error received: " + bind.error().text());
+                    disconnectFromHost();
                 }
             }
             // extensions
@@ -674,19 +725,10 @@ void QXmppOutgoingClient::handleStanza(const QDomElement &nodeRecv)
         d->smId = streamManagementEnabled.id();
         d->canResume = streamManagementEnabled.resume();
         if (streamManagementEnabled.resume() && !streamManagementEnabled.location().isEmpty()) {
-            QRegExp locationRegex("([^:]+)(:[0-9]+)?");
-            if (locationRegex.exactMatch(streamManagementEnabled.location())) {
-                d->resumeHost = locationRegex.cap(0);
-                if (!locationRegex.cap(2).isEmpty())
-                    d->resumePort = locationRegex.cap(2).mid(1).toUShort();
-                else
-                    d->resumePort = 5222;
-            } else {
-                d->resumeHost = QString();
-                d->resumePort = 0;
-            }
+            setResumeAddress(streamManagementEnabled.location());
         }
 
+        d->streamManagementEnabled = true;
         enableStreamManagement(true);
         // we are connected now
         emit connected();
@@ -695,7 +737,9 @@ void QXmppOutgoingClient::handleStanza(const QDomElement &nodeRecv)
         streamManagementResumed.parse(nodeRecv);
         setAcknowledgedSequenceNumber(streamManagementResumed.h());
         d->isResuming = false;
+        d->streamResumed = true;
 
+        d->streamManagementEnabled = true;
         enableStreamManagement(false);
         // we are connected now
         // TODO: The stream was resumed. Therefore, we should not send presence information or request the roster.
@@ -767,6 +811,34 @@ void QXmppOutgoingClient::pingTimeout()
     emit error(QXmppClient::KeepAliveError);
 }
 
+bool QXmppOutgoingClient::setResumeAddress(const QString &address)
+{
+    if (const auto location = parseHostAddress(address);
+        !location.first.isEmpty()) {
+        d->resumeHost = location.first;
+
+        if (location.second > 0) {
+            d->resumePort = location.second;
+        } else {
+            d->resumePort = 5222;
+        }
+        return true;
+    }
+
+    d->resumeHost.clear();
+    d->resumePort = 0;
+    return false;
+}
+
+std::pair<QString, int> QXmppOutgoingClient::parseHostAddress(const QString &address)
+{
+    QUrl url("//" + address);
+    if (url.isValid() && !url.host().isEmpty()) {
+        return { url.host(), url.port() };
+    }
+    return { {}, -1 };
+}
+
 void QXmppOutgoingClientPrivate::sendNonSASLAuth(bool plainText)
 {
     QXmppNonSASLAuthIq authQuery;
