@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Authors:
  *  Manjeet Dahiya
@@ -55,6 +55,8 @@ public:
     bool isAuthenticated() const;
     bool isConnected() const override;
     bool isClientStateIndicationEnabled() const;
+    bool isStreamManagementEnabled() const;
+    bool isStreamResumed() const;
 
     QSslSocket *socket() const { return QXmppStream::socket(); };
     QXmppStanza::Error::Condition xmppStreamError();
@@ -74,7 +76,8 @@ Q_SIGNALS:
     /// This signal is emitted when a message is received.
     void messageReceived(const QXmppMessage &);
 
-    /// This signal is emitted when an IQ is received.
+    /// This signal is emitted when an IQ response (type result or error) has
+    /// been received that was not handled by elementReceived().
     void iqReceived(const QXmppIq &);
 
     /// This signal is emitted when SSL errors are encountered.
@@ -103,7 +106,12 @@ private Q_SLOTS:
     void pingTimeout();
 
 private:
+    bool setResumeAddress(const QString &address);
+    static std::pair<QString, int> parseHostAddress(const QString &address);
+
     friend class QXmppOutgoingClientPrivate;
+    friend class tst_QXmppOutgoingClient;
+
     QXmppOutgoingClientPrivate *const d;
 };
 
