@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Authors:
  *  Manjeet Dahiya
@@ -35,10 +35,11 @@
 
 class QXmppRosterManagerPrivate;
 
+///
 /// \brief The QXmppRosterManager class provides access to a connected client's
 /// roster.
 ///
-/// \note It's object should not be created using it's constructor. Instead
+/// \note Its object should not be created using its constructor. Instead
 /// \c QXmppClient::findExtension<QXmppRosterManager>() should be used to get
 /// the instantiated object of this class.
 ///
@@ -63,7 +64,7 @@ class QXmppRosterManagerPrivate;
 /// roster item changes.
 ///
 /// \ingroup Managers
-
+///
 class QXMPP_EXPORT QXmppRosterManager : public QXmppClientExtension
 {
     Q_OBJECT
@@ -87,19 +88,23 @@ public:
     /// \endcond
 
 public Q_SLOTS:
-    bool acceptSubscription(const QString &bareJid, const QString &reason = QString());
-    bool refuseSubscription(const QString &bareJid, const QString &reason = QString());
-    bool addItem(const QString &bareJid, const QString &name = QString(), const QSet<QString> &groups = QSet<QString>());
+    bool acceptSubscription(const QString &bareJid, const QString &reason = {});
+    bool refuseSubscription(const QString &bareJid, const QString &reason = {});
+    bool addItem(const QString &bareJid, const QString &name = {}, const QSet<QString> &groups = {});
     bool removeItem(const QString &bareJid);
     bool renameItem(const QString &bareJid, const QString &name);
-    bool subscribe(const QString &bareJid, const QString &reason = QString());
-    bool unsubscribe(const QString &bareJid, const QString &reason = QString());
+    bool subscribe(const QString &bareJid, const QString &reason = {});
+    bool unsubscribe(const QString &bareJid, const QString &reason = {});
 
 Q_SIGNALS:
     /// This signal is emitted when the Roster IQ is received after a successful
     /// connection. That is the roster entries are empty before this signal is emitted.
     /// One should use getRosterBareJids() and getRosterEntry() only after
     /// this signal has been emitted.
+    ///
+    /// \note If the previous stream could be resumed, this signal is not
+    /// emitted since QXmpp 1.4. Changes since the last connection are reported
+    /// via the itemAdded(), itemChanged() and itemRemoved() signals.
     void rosterReceived();
 
     /// This signal is emitted when the presence of a particular bareJid and resource changes.
