@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Authors:
  *  Jeremy Lainé
@@ -25,6 +25,9 @@
 #include "QXmppBitsOfBinaryContentId.h"
 #include "QXmppBitsOfBinaryDataList.h"
 #include "QXmppMessage.h"
+#include "QXmppMixInvitation.h"
+
+#include <optional>
 
 #include "util.h"
 #include <QObject>
@@ -63,6 +66,7 @@ private slots:
     void testStanzaIds();
     void testSlashMe_data();
     void testSlashMe();
+    void testMixInvitation();
 };
 
 void tst_QXmppMessage::testBasic_data()
@@ -1076,5 +1080,26 @@ void tst_QXmppMessage::testSlashMe()
     QCOMPARE(msg.slashMeCommandText(), actionText);
 }
 
+void tst_QXmppMessage::testMixInvitation()
+{
+    const QByteArray xml(
+        "<message id=\"f5pp2toz\" to=\"cat@shakespeare.example\" from=\"hag66@shakespeare.example/UUID-h5z/0253\" type=\"normal\">"
+        "<body>Would you like to join the coven?</body>"
+        "<invitation xmlns=\"urn:xmpp:mix:misc:0\">"
+        "<inviter>hag66@shakespeare.example</inviter>"
+        "<invitee>cat@shakespeare.example</invitee>"
+        "<channel>coven@mix.shakespeare.example</channel>"
+        "<token>ABCDEF</token>"
+        "</invitation>"
+        "</message>");
+
+    QXmppMessage message;
+    parsePacket(message, xml);
+
+    QVERIFY(message.mixInvitation());
+
+    serializePacket(message, xml);
+}
+
 QTEST_MAIN(tst_QXmppMessage)
 #include "tst_qxmppmessage.moc"
