@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 The QXmpp developers
+ * Copyright (C) 2008-2021 The QXmpp developers
  *
  * Author:
  *  Jeremy Lainé
@@ -190,14 +190,22 @@ void tst_QXmppSasl::testSuccess()
 
 void tst_QXmppSasl::testClientAvailableMechanisms()
 {
-    QCOMPARE(QXmppSaslClient::availableMechanisms(), QStringList() << "SCRAM-SHA-256"
-                                                                   << "SCRAM-SHA-1"
-                                                                   << "DIGEST-MD5"
-                                                                   << "PLAIN"
-                                                                   << "ANONYMOUS"
-                                                                   << "X-FACEBOOK-PLATFORM"
-                                                                   << "X-MESSENGER-OAUTH2"
-                                                                   << "X-OAUTH2");
+    const QStringList expectedMechanisms = {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 2)
+        "SCRAM-SHA3-512",
+#endif
+        "SCRAM-SHA-512",
+        "SCRAM-SHA-256",
+        "SCRAM-SHA-1",
+        "DIGEST-MD5",
+        "PLAIN",
+        "ANONYMOUS",
+        "X-FACEBOOK-PLATFORM",
+        "X-MESSENGER-OAUTH2",
+        "X-OAUTH2"
+    };
+
+    QCOMPARE(QXmppSaslClient::availableMechanisms(), expectedMechanisms);
 }
 
 void tst_QXmppSasl::testClientBadMechanism()
